unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, IniFiles;

type
  TfmMain = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Button1: TButton;
    Button2: TButton;
    ProjectName: TEdit;
    cmdSelectProject: TButton;
    ResourceName: TStaticText;
    Label3: TLabel;
    BinDir: TEdit;
    OpenDialog: TOpenDialog;
    StaticText1: TStaticText;
    procedure Button2Click(Sender: TObject);
    procedure cmdSelectProjectClick(Sender: TObject);
    procedure ProjectNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    function MakeScriptFile(Project, Resource: string): string;
    procedure Compile(Compiler, Script: string);
  public
    { Public declarations }
  end;

var
  fmMain: TfmMain;

implementation

{$R *.DFM}
{$R VRCver.RES}

procedure TfmMain.Button2Click(Sender: TObject);
begin
  Close;
end;

procedure TfmMain.cmdSelectProjectClick(Sender: TObject);
begin
  if OpenDialog.Execute then ProjectName.Text := OpenDialog.FileName;
end;

procedure TfmMain.ProjectNameChange(Sender: TObject);
var
 buff: string;
begin
  buff := ProjectName.Text;
  buff := Copy(Buff,1,Length(Buff) - 4);
  ResourceName.Caption := buff + 'ver.res';
end;

procedure TfmMain.FormCreate(Sender: TObject);
var
  ini: TIniFile;
begin
  ini := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'VRC.ini');
  BinDir.Text := ini.ReadString('Main','BIN','');
  ProjectName.Text := ini.ReadString('Main','Last','');
  Ini.Free;
end;

procedure TfmMain.FormDestroy(Sender: TObject);
var
  ini: TIniFile;
begin
  ini := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'VRC.ini');
  ini.WriteString('Main','BIN',BinDir.Text);
  ini.WriteString('Main','Last',ProjectName.Text);
  Ini.Free;
end;


procedure TfmMain.Button1Click(Sender: TObject);
var
  BRC, PName, Script: string;
  Include, msg: string;
begin
  BRC := BinDir.Text;
  if BRC[Length(BRC)] <> '\' then BRC := BRC + '\';
  BRC := BRC + 'BRCC32.EXE';
  if not FileExists(BRC) then begin
    MessageDlg('Could not locate the resource compiler: '#13#10 + BRC, mtError,[mbOk],0);
    Exit;
  end;

  PName := ProjectName.Text;
  if not FileExists(PName) then begin
    MessageDlg('Could not locate the project: '#13#10 + PName, mtError,[mbOk],0);
    Exit;
  end;
  Script := MakeScriptFile(PName, ResourceName.Caption);

  Compile(BRC, Script);

  include := ExtractFileName(Script);
  include := Copy(Include,1,length(include) - 3);
  include := include + 'res';

  msg := 'Version Resource created and compiled. Dont forget to add '#13#10'{$R '
         + include + '}'#13#10'to one of your project source files under the ' +
         'implementation section.';
  MessageDlg(msg,mtInformation,[mbOk],0);

end;


function TfmMain.MakeScriptFile(Project, Resource: string): string;
var
 Script, DOF: string;
 f: TextFile;
 ini: TIniFile;
 Local, CodePage, Diff, i: integer;
 hexLocal, hexCodePage: string;
 Major, Minor, Release, Build: integer;
 version: string;
begin
  Script := Copy(Resource,1,Length(Resource) - 3) + 'ver';
  DOF := Copy(Project,1,Length(Project) - 3) + 'dof';
  ini := TIniFile.Create(DOF);

  AssignFile(f,Script);
  Rewrite(f);


  //Get the local
  Local := ini.ReadInteger('Version Info','Locale',0);
  hexLocal := Format('%x', [Local]);
  diff := 4 - Length(hexLocal);
  if diff <> 0 then
    for i := 1 to Diff do hexLocal := '0' + hexLocal;


  //Get the CodePage
  CodePage := ini.ReadInteger('Version Info','CodePage',0);
  hexCodePage := Format('%x', [CodePage]);
  diff := 4 - Length(hexCodePage);
  if diff <> 0 then
    for i := 1 to Diff do hexCodePage := '0' + hexCodePage;

  Major := ini.ReadInteger('Version Info','MajorVer',0);
  Minor := ini.ReadInteger('Version Info','MinorVer',0);
  Release := ini.ReadInteger('Version Info','Release',0);
  Build := ini.ReadInteger('Version Info','Build',0);

  if ini.ReadBool('Version Info','AutoIncBuild',FALSE) then inc(Build);

  version := IntToStr(Major) + '.' + IntToStr(Minor) + '.' + IntToStr(Release) + '.' + IntToStr(Build);


  WriteLn(F,'VS_VERSION_INFO VERSIONINFO');
  WriteLn(F,'  BEGIN');
  WriteLn(F,'      BLOCK "StringFileInfo"');
  WriteLn(F,'      BEGIN');
  WriteLn(F,'           BLOCK "' + hexLocal + hexCodepage + '"');
  WriteLn(F,'           BEGIN');
  WriteLn(F,'                VALUE "CompanyName", "' + ini.ReadString('Version Info Keys', 'CompanyName','')+ '\0"');
  WriteLn(F,'                VALUE "FileDescription", "' + ini.ReadString('Version Info Keys', 'FileDescription','')+ '\0"');
  WriteLn(F,'                VALUE "FileVersion", "' + version + '\0"');
  WriteLn(F,'                VALUE "InternalName", "' + ini.ReadString('Version Info Keys', 'InternalName','')+ '\0"');
  WriteLn(F,'                VALUE "LegalCopyright", "' + ini.ReadString('Version Info Keys', 'LegalCopyright','Copyright  1998')+ '\0"');
  WriteLn(F,'                VALUE "LegalTrademarks", "' + ini.ReadString('Version Info Keys', 'LegalTrademarks','')+ '\0"');
  WriteLn(F,'                VALUE "OriginalFilename", "' + ini.ReadString('Version Info Keys', 'OriginalFilename','')+ '\0"');
  WriteLn(F,'                VALUE "ProductName", "' + ini.ReadString('Version Info Keys', 'ProductName','')+ '\0"');
  WriteLn(F,'                VALUE "ProductVersion", "' + ini.ReadString('Version Info Keys', 'ProductVersion','')+ '\0"');
  WriteLn(F,'                VALUE "Comments", "' + ini.ReadString('Version Info Keys', 'Comments','')+ '\0"');
  WriteLn(F,'           END');
  WriteLn(F,'      END');
  WriteLn(F,'  END');

  System.Close(f);
  ini.Free;
  result := script;
end;

procedure TfmMain.Compile(Compiler, Script: string);
var
  CommandLine: PChar;
  bStat: BOOL;
  pi: TProcessInformation;
  si: TStartupInfo;
begin

  //Prepair the process
  FillMemory( @si, sizeof( TStartupInfo ), 0 );

  with si do
    begin
    cb := sizeof( TStartupInfo );
    lpReserved := nil;
    lpTitle := nil;
    dwFlags := STARTF_USESHOWWINDOW;
    wShowWindow := 0;
    end;

  CommandLine := StrAlloc(Length(Compiler + ' ' + Script) + 1);
  StrPCopy(CommandLine, Compiler + ' ' + Script);

  bStat := CreateProcess
    (
    nil,  // address of module name
    CommandLine, // address of command line
    nil,  // address of process security attributes
    nil,  // address of thread security attributes
    false,  // new process inherits handles
    REALTIME_PRIORITY_CLASS,  // creation flags
    nil,  // address of new environment block
    nil,  // address of current directory name
    si, // address of STARTUPINFO
    pi  // address of PROCESS_INFORMATION
    );
  StrDispose(Commandline);



end;

end.
